"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

import contextlib
import re
import socket
import subprocess
import time

import requests
import yaml

from . import config, log


def check_docker_network():
    log.get_logger().debug(f'Checking for docker network {config.get_dc_network()}')
    exit_code = subprocess.call(
        ['docker', 'network', 'inspect', config.get_dc_network()],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )
    if exit_code:
        log.get_logger().info(f'Creating docker network {config.get_dc_network()}')
        subprocess.check_call(
            ['docker', 'network', 'create', config.get_dc_network()],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )


def get_open_ports(container_name):
    """
    Retrieves the open ports on a container.
    :param container_name:
    :return: The open ports as a dictionary of container ports to host ports
    """
    try:
        lines = subprocess.check_output(['docker', 'port', container_name]).splitlines()
        ports = {}
        for line in lines:
            match = re.match(r'^(\d+)/tcp -> 0.0.0.0:(\d+)$', line.strip().decode('utf-8'))
            if not match:
                continue
            ports[int(match.group(1))] = int(match.group(2))
        return ports
    except subprocess.CalledProcessError:
        log.get_logger().warning(
            f'Could not find open ports for {container_name}, please ensure it is configured correctly'
        )
        return []


def is_port_open(port):
    """
    Checks if the port is open on localhost by creating a socket connection to it.
    :param port: The port as a number
    :return: True if open, false otherwise
    """
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        return sock.connect_ex(('127.0.0.1', port)) == 0


def _is_path_responding(port: int, path: str) -> bool:
    with contextlib.suppress(Exception):
        return requests.get(f'http://localhost:{port}{path}').status_code == 200


def check_port(container_name: str, port: int, path: str) -> None:
    total_time = 0.1
    while not _is_path_responding(port, path):
        message = f'Container {container_name} is not yet up, sleeping...'
        if (total_time % 3) == 0:
            log.get_logger().info(message)
        else:
            log.get_logger().debug(message)
        total_time += 0.1
        time.sleep(0.1)


def read_services_from_dc(docker_compose_path: str):
    with open(docker_compose_path, encoding='utf8') as fobj:
        data = yaml.safe_load(fobj)
        services = data.get('services', {})
        return services.keys()



"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

import argparse
import os
from typing import Callable, List, Optional, Sequence

from . import commands, config, log
from .data import Project, Service
from .exceptions import KnownException


def _get_print_help_func(parser):
    # pylint: disable=unused-argument
    def print_help(args: argparse.Namespace):
        parser.print_help()
        return os.EX_OK

    return print_help


def _parse_args(configure_parsers: Optional[Callable], args: Optional[Sequence[str]]) -> argparse.Namespace:
    # pylint: disable=too-many-locals

    # Get all projects defined in the configuration
    projects = Project.find_all()

    # Parse arguments
    parser = argparse.ArgumentParser(
        description='Coordinates running projects and services through docker compose.',
    )
    parser.add_argument(
        '-p', '--projects-dev',
        action='append',
        default=[],
        dest='dev_project_names',
        # Only projects with actual directories can be developed
        choices=list(project.name for project in projects if project.directory),
        help="The projects to develop locally, does not pull upstream images for them.",
    )
    parser.add_argument(
        "-t", "--tag",
        dest="tag",
        default='latest',
        choices=['latest', 'dev', 'stage', 'prod'],
        help="The docker tag to use for upstream images, defaults to latest (dev is equivalent to latest).",
    )
    parser.add_argument(
        "-d", "--debug",
        dest="debug",
        action="store_true",
        help="Enable debug output.",
    )
    parser.set_defaults(
        func=_get_print_help_func(parser),
    )

    # Build arguments for enabled/disabled service dynamically
    for service in Service.find_has_enable_flag():
        parser.add_argument(
            f"--enable-{service.name.replace('_', '-')}",
            dest=f"enable_{service.name}",
            action="store_true",
            help=f"Enables the optional {service.name} service from the {service.project_name} "
                 f"project, disabled by default",
        )
    for service in Service.find_has_disable_flag():
        parser.add_argument(
            f"--disable-{service.name.replace('_', '-')}",
            dest=f"disable_{service.name}",
            action="store_true",
            help=f"Disables the optional {service.name} service from the {service.project_name} "
                 f"project, enabled by default",
        )

    subparsers = parser.add_subparsers(title='Commands', help='commands')

    # Help shortcut
    # help_parser = subparsers.add_parser(
    #     'help',
    #     aliases=['h'],
    #     help='Prints help for commands and usages'
    # )

    # Checkout
    checkout_parser = subparsers.add_parser(
        'checkout',
        aliases=['co'],
        help='Clones and/or updates repositories for all specified projects'
    )
    checkout_parser.set_defaults(
        func=commands.run_checkout,
    )
    checkout_parser.add_argument(
        "-a", "--all-projects",
        dest="all_projects",
        action="store_true",
        help="Checkout/clone all projects.",
    )
    checkout_parser.add_argument(
        '-e', '--extra-remote',
        dest='extra_remotes',
        action='append',
        nargs='*',
        default=[],
        help="Add a remote when cloning/checking out. Each flag can take 1-2 params, 1 param should be the space "
             "the remote exists in github. If a 2nd param is specified it is an optional name for the remote."
    )

    # Repo Status
    repo_status_parser = subparsers.add_parser(
        'repo-status',
        aliases=['rs'],
        help='gets the status of the repos associated with this control script'
    )
    repo_status_parser.set_defaults(
        func=commands.get_repo_status,
    )
    repo_status_parser.add_argument(
        "-a", "--all-projects",
        dest="all_projects",
        action="store_true",
        help="Get the status of all projects.",
    )

    # Docker Status
    # Init
    init_parser = subparsers.add_parser(
        'init',
        help='Generates docker-compose templates and copies configuration, but does not run any commands'
    )
    init_parser.set_defaults(
        func=commands.run_dc_init,
    )

    # Docker compose
    dc_parser = subparsers.add_parser(
        'docker-compose',
        aliases=['dc'],
        help='Generates docker-compose templates, copies configuration, and runs any docker-compose command'
    )
    dc_parser.set_defaults(
        func=commands.run_docker_compose,
    )
    dc_parser.add_argument(
        'docker_compose_args',
        nargs='*',
        help="The arguments to pass directly to docker-compose.",
    )

    # Docker compose aliases
    build_parser = subparsers.add_parser(
        'build',
        help='Alias for the "dc build" command'
    )
    build_parser.add_argument(
        "--no-cache",
        dest="no_cache",
        action="store_true",
        help="Do not use cache when building the image.",
    )
    build_parser.add_argument(
        "-a", "--all-projects",
        dest="all_projects",
        action="store_true",
        help="Builds all projects instead of just those specified or by using the current directory,"
             "this is assumed if no projects are specified.",
    )
    build_parser.set_defaults(
        func=commands.run_dc_build,
    )
    config_parser = subparsers.add_parser(
        'config',
        help='Alias for the "dc config" command'
    )
    config_parser.set_defaults(
        func=commands.run_dc_config,
    )
    down_parser = subparsers.add_parser(
        'down',
        help='Alias for the "dc down" command'
    )
    down_parser.set_defaults(
        func=commands.run_dc_down,
    )
    pull_parser = subparsers.add_parser(
        'pull',
        help='Alias for the "dc pull" command'
    )
    pull_parser.set_defaults(
        func=commands.run_dc_pull,
    )
    rm_parser = subparsers.add_parser(
        'rm',
        help='Alias for the "dc rm --force" command'
    )
    rm_parser.set_defaults(
        func=commands.run_dc_rm,
    )
    stop_parser = subparsers.add_parser(
        'stop',
        help='Alias for the "dc stop" command'
    )
    stop_parser.set_defaults(
        func=commands.run_dc_stop,
    )
    up_parser = subparsers.add_parser(
        'up',
        help='Alias for the "dc up" command'
    )
    up_parser.set_defaults(
        func=commands.run_dc_up,
    )
    up_detach_parser = subparsers.add_parser(
        'up-detach',
        help='Alias for the "dc up --detach" command'
    )
    up_detach_parser.set_defaults(
        func=commands.run_dc_up_detach,
    )
    up_recreate_parser = subparsers.add_parser(
        'up-recreate',
        help='Alias for the "dc up --force-recreate" command'
    )
    up_recreate_parser.set_defaults(
        func=commands.run_dc_up_recreate,
    )

    # Do not allow to pull configuration unless there is a project for config
    config_service_target = Service.find_config()
    if config_service_target:
        pull_config_parser = subparsers.add_parser(
            'pull-config',
            help='Alias for the "dc pull" command'
        )
        pull_config_parser.set_defaults(
            func=commands.run_dc_pull_config,
        )

    if configure_parsers:
        configure_parsers(parser, subparsers)

    # Parse arguments
    return parser.parse_args(args)


def _validate_args(args: argparse.Namespace) -> int:
    """
    Validates command line arguments for situations that are too complex for argparse.
    When this is called, all developed projects must already be added.
    :param args: the CLI arguments parsed by argparse
    :return: the exit code (or 0 on successful validation)
    """
    # Configure extra remotes
    if 'extra_remotes' in args.__dict__ and len(args.extra_remotes) > 0:
        for remote in args.extra_remotes:
            if len(remote) > 2 or len(remote) < 1:
                log.get_logger().error(
                    'extra remotes parameters must take either 1 or 2 parameters for each specification.'
                )
                log.get_logger().error('example : -e origin bob')
                log.get_logger().error(
                    '      where origin is the remote name and bob is the space the repo is forked in'
                )
                return os.EX_USAGE

    # Make sure the config service is not disabled at the same time it is being developed
    config_service_target = Service.find_config()
    # pylint: disable=too-many-boolean-expressions
    if config_service_target and config_service_target.project_name in args.dev_project_names and \
            not config_service_target.is_enabled(args):
        log.get_logger().error(
            f'To use the config service, you must not be developing on the '
            f'{config_service_target.project_name} project.'
        )
        return os.EX_USAGE
    return os.EX_OK


def _detect_current_project(dev_project_names: List[str]) -> None:
    dir_name = os.path.basename(os.getcwd())
    base_dir_name = os.path.basename(config.get_base_dir())
    if dir_name != base_dir_name:
        for project in Project.find_all():
            if project.directory == dir_name and project.name not in dev_project_names:
                log.get_logger().info(f'Assuming development for project {project.name}')
                dev_project_names.append(project.name)


def run(base_dir: str, configure_parsers: Callable = None, args: Optional[Sequence[str]] = None) -> int:
    """
    Runs the CLI control command, including parsing arguments, etc.
    :param base_dir: The base directory of the control project
    :param configure_parsers: An optional method to configure additional parameters or arguments on the parser.
                              It receives two arguments, the top-level parser and the subparser for commands.
    :param args: Optional arguments to use instead of sys.argv.
    :return: The exit code
    """
    # Initialize config based on the base dir
    config.initialize(base_dir)

    args = _parse_args(configure_parsers, args)

    # Initialize logging
    log.init_logger(args.debug)

    _detect_current_project(args.dev_project_names)
    errno = _validate_args(args)
    if errno:
        return errno

    try:
        return args.func(args)
    except KnownException as exc:
        log.get_logger().error(str(exc))
        return os.EX_SOFTWARE
    except Exception as exc:  # pylint:disable=broad-except
        log.get_logger().error(f'Encountered an unexpected failure ({exc.__class__}): {exc}')
        if args.debug:
            raise
        return os.EX_SOFTWARE



"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

import argparse
import os
from typing import List

import jinja2
import yaml

from . import config, docker_utils, log
from .data import Project, Service
from .exceptions import KnownException


def _check_required_options():
    log.get_logger().info(f'Validating config file ({config.ENV_FILE})')
    if not os.path.exists(config.get_env_file_path()):
        raise KnownException(f'Please create {config.get_env_file_path()} containing env vars for project config, '
                             f'this will be used to configure each project. The {config.ENV_FILE}.example '
                             'file may be used as a template')
    with open(config.get_env_file_path(), 'r', encoding='utf8') as fobj:
        contents = fobj.read()
    required_options = config.get_required_options()
    for option in required_options:
        if option not in contents:
            raise KnownException(f'The {config.get_env_file_path()} file must include the {option} option')


def _check_project_directories(projects: List[Project]) -> None:
    if len(projects) == 0:
        log.get_logger().debug('No projects specified, not validating directories')
        return
    log.get_logger().info(f'Validating directories are present for {len(projects)} projects(s)')
    for project in projects:
        if not os.path.exists(project.path) or not os.path.isdir(project.path):
            raise KnownException(f'No directory found for the "{project.name}" project at {project.path}, '
                                 'please check the repository')


def _set_dynamic_options(args: argparse.Namespace) -> None:
    # pylint: disable=too-many-branches

    with open(config.get_env_file_path(), 'r', encoding='utf8') as fobj:
        env_file_contents = fobj.read()
    new_contents = []
    lines_changed = False

    enabled_status_by_service = {}
    services_by_dynamic_option = {}
    for service in Service.find_all():
        enabled_status_by_service[service.name] = service.is_enabled(args)
        for option in service.dynamic_options.keys():
            services_by_dynamic_option[option] = service
    dynamic_options_seen = {}
    for option in services_by_dynamic_option:
        dynamic_options_seen[option] = False

    for line in env_file_contents.splitlines():
        option_considered = False
        for option, service in services_by_dynamic_option.items():
            if not line.startswith(f'{option}='):
                continue
            option_considered = True
            dynamic_options_seen[option] = True
            if enabled_status_by_service.get(service.name, True):
                value = service.dynamic_options[option].get('enabled')
            else:
                value = service.dynamic_options[option].get('disabled')

            if value:
                new_line = f'{option}={value}'
                if line != new_line:
                    log.get_logger().info(f'Replacing dynamic option for {option} in {config.ENV_FILE} with {value}')
                    lines_changed = True
                else:
                    log.get_logger().info(f'Keeping existing dynamic option for {option} in {config.ENV_FILE}')
                new_contents.append(new_line)
            else:
                log.get_logger().info(f'Removing dynamic option for {option} in {config.ENV_FILE}')
                lines_changed = True

        # If no dynamic options are in the line, append it
        if not option_considered:
            new_contents.append(line)

    for option, service in services_by_dynamic_option.items():
        if not dynamic_options_seen[option]:
            if enabled_status_by_service.get(service.name, True):
                value = service.dynamic_options[option].get('enabled')
            else:
                value = service.dynamic_options[option].get('disabled')
            if value:
                log.get_logger().info(f'Adding dynamic option for {option} in {config.ENV_FILE} with value {value}')
                new_contents.append(f'{option}={value}')
                lines_changed = True

    if lines_changed:
        with open(config.get_env_file_path(), 'w', encoding='utf8') as fobj:
            fobj.write('\n'.join(new_contents))


def _link_config(projects: List[Project]):
    if len(projects) == 0:
        log.get_logger().debug(f'No projects specified, not linking {config.ENV_FILE}')
        return
    log.get_logger().info(f'Linking base {config.ENV_FILE} to {len(projects)} projects(s)')
    for project in projects:
        env_project_path = os.path.join(project.path, config.ENV_FILE)
        if not os.path.exists(env_project_path):
            os.symlink(config.get_env_file_path(), env_project_path)
        elif os.path.islink(env_project_path):
            # nothing to do if it already exists
            pass
        else:
            os.remove(env_project_path)
            os.symlink(config.get_env_file_path(), env_project_path)
        log.get_logger().debug(f'Linked in {project.path}')


def _render_docker_compose_file(project_dir, template_config):
    if not os.path.exists(os.path.join(project_dir, config.DOCKER_COMPOSE_TEMPLATE)):
        log.get_logger().debug(
            f'No {config.DOCKER_COMPOSE_TEMPLATE} detected in the {project_dir} directory, skipping generation'
        )
        return
    try:
        env = jinja2.Environment(loader=jinja2.FileSystemLoader(project_dir))
        template = env.get_template(config.DOCKER_COMPOSE_TEMPLATE)
        output = template.render(**template_config)
        with open(os.path.join(project_dir, config.DOCKER_COMPOSE_FILE), 'w', encoding='utf8') as fobj:
            fobj.write(output)
    except jinja2.exceptions.TemplateSyntaxError as exc:
        # pylint: disable=raise-missing-from
        raise KnownException(
            f'Could not render {project_dir}/{config.DOCKER_COMPOSE_TEMPLATE}: {exc.message} on line {exc.lineno}'
        )


def _generate_docker_compose(args: argparse.Namespace, dev_projects: List[Project], tag: str):
    registry = config.get_registry(tag)
    log.get_logger().info(f'Generating docker-compose.yml with {len(dev_projects)} project(s) and tag "{tag}"')

    services = {}
    dev_project_names = list(project.name for project in dev_projects)
    projects = Project.find_all()

    enabled_services = {}
    for project in projects:
        for service in project.services:
            enabled_services[service.name] = service.is_enabled(args)

    for project in projects:
        if project.name in dev_project_names:
            continue
        for service in project.services:
            if not enabled_services.get(service.name):
                continue
            data = service.get_dc_data(registry, tag)
            # Add env file and network dynamically
            if 'env_file' not in data:
                data['env_file'] = [f'./{config.ENV_FILE}']
            if 'networks' not in data:
                data['networks'] = [config.get_dc_network()]
            services[service.dc_name] = data

    # Write out base docker compose
    docker_compose_data = {
        'version': '3',
        'networks': {
            config.get_dc_network(): {
                'external': True,
            },
        },
        'services': services,
    }
    docker_compose_data.update(config.get_dc_data())
    with open(config.get_docker_compose_path(), 'w', encoding='utf8') as fobj:
        yaml.safe_dump(docker_compose_data, fobj)

    template_config = {
        'dev_project_names': dev_project_names,
        'enabled_services': enabled_services,
        'tag': tag,
        'registry': registry,
        'network': config.get_dc_network(),
        'core_prefix': config.get_service_prefix('core'),
        'service_prefix': config.get_service_prefix(),
    }
    for project in dev_projects:
        _render_docker_compose_file(project.path, template_config)


def init_docker_compose(args: argparse.Namespace, dev_projects: List[Project], no_network: bool = False) -> None:
    if not no_network:
        docker_utils.check_docker_network()
    _check_project_directories(dev_projects)
    _check_required_options()
    _set_dynamic_options(args)
    _link_config(dev_projects)
    _generate_docker_compose(args, dev_projects, args.tag)



"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

import logging


# Global vars
_LOGGER: logging.Logger = None


def init_logger(debug: bool) -> None:
    # pylint: disable=global-statement
    global _LOGGER

    log_level = logging.INFO
    if debug:
        log_level = logging.DEBUG
    log_handler = logging.StreamHandler()
    log_handler.setLevel(log_level)

    logger = logging.getLogger()
    logger.setLevel(log_level)
    logger.addHandler(log_handler)
    _LOGGER = logger


def get_logger() -> logging.Logger:
    if not _LOGGER:
        init_logger(True)
    return _LOGGER



"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""


class KnownException(Exception):
    """
    Marker class for exceptions that should not have a full stacktrack printed out.
    """



"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

from .cli import run
from .commands import call_commands
from .data import Project, Service
from .exceptions import KnownException



"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

import os
from functools import lru_cache
from typing import Any, Dict, Iterable, List, Optional, Union

import yaml

from .exceptions import KnownException


# Static vars
CONFIG_FILE = 'config.yml'
DOCKER_COMPOSE_TEMPLATE = 'docker-compose-template.yml'
DOCKER_COMPOSE_FILE = 'docker-compose.yml'
ENV_FILE = 'docker-compose.env'

# Global vars
_BASE_DIR = os.path.dirname(__file__)


def initialize(base_dir: str) -> None:
    """
    Sets the base dir for all further operations, this should be called first
    """
    # pylint: disable=global-statement
    global _BASE_DIR
    _BASE_DIR = base_dir


def get_base_dir() -> str:
    return _BASE_DIR


# The following methods are cached so that we only retrieve/validate them once (config cannot be changed mid-run)
@lru_cache()
def get_env_file_path() -> str:
    return os.path.join(_BASE_DIR, ENV_FILE)


@lru_cache()
def get_docker_compose_path() -> str:
    return os.path.join(_BASE_DIR, DOCKER_COMPOSE_FILE)


@lru_cache()
def _get_config() -> Dict[str, Any]:
    config_path = os.path.join(_BASE_DIR, CONFIG_FILE)
    if not os.path.exists(config_path):
        raise KnownException(f'Config file {config_path} does not exist, please copy and modify example')
    with open(config_path, 'r', encoding='utf8') as config_file:
        try:
            config = yaml.safe_load(config_file)
        except Exception:
            # pylint: disable=raise-missing-from
            raise KnownException(f'Config file {config_path} is not valid YAML, please copy and modify example')
    if not config:
        raise KnownException(f'Config file {config_path} is invalid, please copy and modify example')

    # Validate configuration
    prefixes = config.get('prefixes')
    if not prefixes or not prefixes.get('service') or not prefixes.get('core'):
        raise KnownException(
            f'Config file {config_path} has an invalid "prefixes" configuration, please see example'
        )
    dc_config = config.get('docker-compose')
    if not dc_config or not dc_config.get('network') or \
            not dc_config.get('project') or not dc_config.get('registry'):
        raise KnownException(
            f'Config file {config_path} has an invalid "docker-compose" configuration, please see example'
        )
    projects = config.get('projects')
    if not projects or not isinstance(projects, dict):
        raise KnownException(
            f'Config file {config_path} has an invalid "projects" configuration, please see example'
        )
    for project_name in projects:
        project_data = projects[project_name]
        # pylint: disable=too-many-boolean-expressions
        if not project_data or not isinstance(project_data, dict) or 'directory' not in project_data or \
                'repository' not in project_data or 'services' not in project_data or \
                not isinstance(project_data.get('services'), list):
            raise KnownException(
                f'Config file {config_path} has an invalid "projects.{project_name}" '
                f'configuration, please see example'
            )
        for service_data in project_data.get('services'):
            if not isinstance(service_data, dict) or 'name' not in service_data:
                raise KnownException(
                    f'Config file {config_path} has an invalid "projects.{project_name}.services" '
                    f'entry, please see example'
                )
    return config


def get_service_prefix(service_type: str = 'service') -> str:
    """
    Gets a prefix for the given service type.
    :param service_type: The service type, defaults to "service"
    :return: The service prefix, or none if not defined
    """
    return _get_config()['prefixes'].get(service_type)


def get_target_service(target_name: str, optional: bool = False) -> Optional[str]:
    """
    Gets a target service container name from the config.
    :param target_name: The target name
    :param optional: If true, no exception will be raised if the target service is not present or configured
    :return:
    """
    config = _get_config()
    if not config.get('target-services') or target_name not in config['target-services']:
        if optional:
            return None
        raise KnownException(f'Target service {target_name} is not defined, please define "target-services"')
    return config['target-services'][target_name]


def get_project_config() -> Dict[str, dict]:
    return _get_config()['projects']


def get_dc_project() -> str:
    return _get_config()['docker-compose'].get('project')


def get_dc_network() -> str:
    return _get_config()['docker-compose'].get('network')


def get_dc_build_args() -> Optional[Union[Dict[str, str], List[str]]]:
    return _get_config()['docker-compose'].get('build-args')


def get_required_options() -> Iterable[str]:
    return _get_config().get('required-options', [])


def get_tags() -> Iterable[str]:
    return _get_config()['docker-compose'].get('tags', ['latest'])


def get_registry(tag: str) -> str:
    dc_config = _get_config()['docker-compose']
    return dc_config.get('registries-by-tag', {}).get(tag, dc_config['registry'])


def get_dc_data() -> dict:
    dc_config = _get_config()['docker-compose']
    data = {}
    for key, value in dc_config.items():
        if key in ('build-args', 'tags', 'registries-by-tag', 'registry', 'project', 'network'):
            continue
        data[key] = value
    return data
